;  The WOZ Monitor for the Apple 1
;  Written by Steve Wozniak in 1976

;  Ported to the Vic-20 in 2019

; Page 0 Variables

XAML            = $0           ;  Last "opened" location Low
XAMH            = $1           ;  Last "opened" location High
STL             = $2           ;  Store address Low
STH             = $3           ;  Store address High
L               = $4           ;  Hex value parsing Low
H               = $5           ;  Hex value parsing High
YSAV            = $6           ;  Used to see if hex value is given
MODE            = $A           ;  $00=XAM, $7F=STOR, $AE=BLOCK XAM


IN              = $033C         ;  Input buffer to $03BB


*=$6000


RESET           CLD             ; Clear decimal arithmetic mode.
                CLI
                LDY #$7f        ; Set up variable
                lda $cb         ; get C= keyboard matrix
NOTCR           CMP #7          ; DEL?
                BEQ BACKSPACE   ; Yes.
                CMP #48         ; Q? =STOP
                BNE NOTEND
                JMP ENDIT       ; Yes.
NOTEND          INY             ; Advance text index.
                BPL NEXTCHAR    ; Auto ESC if > 127.
ESCAPE          LDA #"/"         ; "/".
                JSR ECHO        ; Output it.
GETLINE         LDA #$D        ; CR.
                TAX
                JSR ECHO        ; Output it.
                LDY #$01        ; Initialize text index.
BACKSPACE       DEY             ; Back up text index.
                BMI GETLINE     ; Beyond start of line, reinitialize.
NEXTCHAR        LDA $cb         ; Key ready?
                CMP $3b
                BEQ NEXTCHAR    ; Loop if same as last character
                sta $3b
                CMP #64         ; =no key
                BEQ NEXTCHAR    ; Loop until ready.
                TAX
                LDA $EC5E,X      ; get character from unshifted decode table
                STA IN,Y        ; Add to text buffer.
                JSR ECHO        ; Display PETSCII character.
                TXA             ; keyboard code (not PETSCII)
                CMP #15         ; CR?
                BNE NOTCR       ; No.
                LDY #$FF        ; Reset text index.
                LDA #$00        ; For XAM mode.
                TAX             ; 0->X.
SETMODE         ASL
SETSTOR         ASL             ; Leaves $7B if setting STOR mode.
                STA MODE        ; $00=XAM $7B=STOR $B8=BLOK XAM
BLSKIP          INY             ; Advance text index.
NEXTITEM        LDA IN,Y        ; Get character.
                CMP #$D         ; CR?
                BEQ GETLINE     ; Yes, done this line.
                CMP #46         ; "."?
                BCC BLSKIP      ; Skip delimiter.
                BEQ SETMODE     ; Yes. Set STOR mode.
                CMP #58         ; ":"?
                BEQ SETSTOR     ; Yes. Set STOR mode.
                CMP #82         ; "R"?
                BEQ RUN         ; Yes. Run user program.
                STX L           ; $00-> L.
                STX H           ; and H.
                STY YSAV        ; Save Y for comparison.
NEXTHEX         LDA IN,Y        ; Get character for hex test.
                EOR #$30        ; Map digits to $0-9.
                CMP #$0A        ; Digit?
                BCC DIG         ; Yes.
                ADC #$88        ; Map letter "A"-"F" to $FA-FF.
                CMP #$FA        ; Hex letter?
                BCC NOTHEX      ; No, character not hex.
DIG             ASL
                ASL             ; Hex digit to MSD of A.
                ASL
                ASL
                LDX #$04        ; Shift count.
HEXSHIFT        ASL             ; Hex digit left, MSB to carry.
                ROL L           ; Rotate into LSD.
                ROL H           ;  Rotate into MSD’s.
                DEX             ; Done 4 shifts?
                BNE HEXSHIFT    ; No, loop.
                INY             ; Advance text index.
                BNE NEXTHEX     ; Always taken. Check next char for hex.
NOTHEX          CPY YSAV        ; Check if L, H empty (no hex digits).
                BEQ ESCAPE      ; Yes, generate ESC sequence.
                BIT MODE        ; Test MODE byte.
                BVC NOTSTOR     ;  B6=0 STOR 1 for XAM & BLOCK XAM
                LDA L           ; LSD’s of hex data.
                STA (STL,X)     ; Store at current ‘store index’.
                INC STL         ; Increment store index.
                BNE NEXTITEM    ; Get next item. (no carry).
                INC STH         ; Add carry to ‘store index’ high order.
TONEXTITEM      JMP NEXTITEM    ; Get next command item.
RUN             JMP (XAML)      ; Run at current XAM index.
NOTSTOR         BMI XAMNEXT     ; B7=0 for XAM, 1 for BLOCK XAM.
                LDX #$02        ; Byte count.
SETADR          LDA L-1,X       ; Copy hex data to
                STA STL-1,X     ; ‘store index’.
                STA XAML-1,X    ; And to ‘XAM index’.
                DEX             ; Next of 2 bytes.
                BNE SETADR      ; Loop unless X=0.
NXTPRNT         BNE PRDATA      ; NE means no address to print.
                LDA #$D        ; CR.
                JSR ECHO        ; Output it.
                LDA XAMH        ; ‘Examine index’ high-order byte.
                JSR PRBYTE      ; Output it in hex format.
                LDA XAML        ; Low-order ‘examine index’ byte.
                JSR PRBYTE      ; Output it in hex format.
                LDA #58    ; ":".
                JSR ECHO        ; Output it.
PRDATA          LDA (XAML,X)    ; Get data byte at ‘examine index’.
                JSR PRBYTE      ; Output it in hex format.
XAMNEXT         STX MODE        ; 0->MODE (XAM mode).
                LDA XAML
                CMP L           ; Compare ‘examine index’ to hex data.
                LDA XAMH
                SBC H
                BCS TONEXTITEM  ; Not less, so no more data to output.
                INC XAML
                BNE MOD8CHK     ; Increment ‘examine index’.
                INC XAMH
MOD8CHK         LDA XAML        ; Check low-order ‘examine index’ byte
                AND #$07        ; For MOD 8=0
                BPL NXTPRNT     ; Always taken.
PRBYTE          PHA             ; Save A for LSD.
                LSR
                LSR
                LSR             ; MSD to LSD position.
                LSR
                JSR PRHEX       ; Output hex digit.
                PLA             ; Restore A.
PRHEX           AND #$0F        ; Mask LSD for hex print.
                ORA #$30    ; Add "0".
                CMP #$3A        ; Digit?
                BCC ECHO        ; Yes, output it.
                ADC #$06        ; Add offset for letter.
ECHO
                jsr $e742       ; C= Basic char out to screen
ENDIT           RTS             ; Return.

DSPOUT  byte 0,0
